/* 
    qasn1 - serializertest - unit tests for serializer class of qasn1 library
    Copyright(C) 2010 - Andrius da Costa Ribas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtCore/QDebug>
#include <QtCore/QObject>
#include <QtTest/QtTest>

#include "Asn1BerParser.h" // for the two ways test
#include "Asn1BerSerializer.h"

#include "serializertest.h"

void Asn1BerSerializerTest::initTestCase()
{
    qDebug() << "startup";
}

void Asn1BerSerializerTest::testPrimitive()
{
    Asn1BerNode* node = new Asn1BerNode();
    QByteArray subject;
    
    node->setTag(16);
    node->setClass(CLASS_CONTEXT);
    node->setVal(QByteArray("ab"));
    
    Asn1BerSerializer* serializer = new Asn1BerSerializer(node, &subject);
    
    serializer->serializeAllTree();
    
    qDebug() << subject;
    QVERIFY((unsigned char) subject.at(0) == 0x90); // tag + class + primitive
    QVERIFY((unsigned char) subject.at(1) == 0x02); // len = 2
    QVERIFY( subject.at(2) == 'a');
    QVERIFY( subject.at(3) == 'b');
    QVERIFY( subject.size() == 4);
}

void Asn1BerSerializerTest::testBigTag()
{
    Asn1BerNode* node = new Asn1BerNode;
    QByteArray subject;
    
    node->setTag(1048608); // same value used on parser
    node->setClass(CLASS_CONTEXT);
    node->setVal("a");
    
    Asn1BerSerializer* serializer = new Asn1BerSerializer(node, &subject);
    
    serializer->serializeAllTree();
    
    qDebug() << subject;
    
    QVERIFY((unsigned char) subject.at(0) == 0x9F );  // class + primitive + (last five bits = 1)
    QVERIFY((unsigned char) subject.at(1) == 0xC0 ); // first bit = 1 + seven bits encoding tag
    QVERIFY((unsigned char) subject.at(2) == 0x80 ); // first bit = 1 + seven bits encoding tag
    QVERIFY((unsigned char) subject.at(3) == 0x20 ); // first bit = 0 + seven bits encoding tag
    QVERIFY((unsigned char) subject.at(4) == 0x01 ); // length = 1
    QVERIFY(subject.at(5) == 'a' ); // value
    
    QVERIFY( subject.size() == 6 );
}

void Asn1BerSerializerTest::testBigLen()
{
    Asn1BerNode* node = new Asn1BerNode;
    QByteArray subject;
    QByteArray auxValue;
    
    node->setTag(1);
    node->setClass(CLASS_UNIVERSAL);
    
    for ( register int i = 0; i < 128; i++ )
    {
        auxValue.append("z");
    }
    
    node->setVal(auxValue);
    
    Asn1BerNode* node2 = node->clone();
    
    node->setNext(node2);
    node2->setPrev(node);
    
    Asn1BerSerializer* serializer = new Asn1BerSerializer(node2, &subject);
    
    serializer->serializeAllTree();
    
    QVERIFY( subject.size() == 262 );
    QVERIFY( (unsigned char) subject.at(0) == 1 );
    QVERIFY( (unsigned char) subject.at(1) == 0x81 );
    QVERIFY( (unsigned char) subject.at(2) == 0x80 );
    QVERIFY( subject.at(3) == 'z' );
    QVERIFY( (unsigned char) subject.at(131) == 1 );
}

void Asn1BerSerializerTest::twoWaysTest()
{
    // please note that a BER file can have more than one possibility to encode the same data
    // in this case the serializer shall produce the same data previously parsed
    
    QFile* inFile = new QFile("test_1.ber");
    inFile->open(QIODevice::ReadOnly);
    
    Asn1BerParser* parser = new Asn1BerParser(inFile);
    
    Asn1BerNode* node = parser->parse();
    
    node = node->next()->firstChild()->next(); // serializer may be able to find the first node :)
    
    QByteArray subject;
    
    Asn1BerSerializer* serializer = new Asn1BerSerializer(node, &subject);
    
    serializer->serializeAllTree();
    
    inFile->seek(0);
    
    QVERIFY(inFile->readAll() == subject);
}

void Asn1BerSerializerTest::testConstructed()
{
    Asn1BerNode* node = new Asn1BerNode;
    
    node->setTag(16);
    node->setClass(CLASS_UNIVERSAL);
    node->setCodingComplex();
    
    Asn1BerNode* child = new Asn1BerNode(node);
    child->setTag(1);
    child->setClass(CLASS_UNIVERSAL);
    child->setVal("a");
    
    Asn1BerNode* child2 = child->clone();
    child->setNext(child2);
    child2->setPrev(child);
    
    QByteArray subject;
    
    Asn1BerSerializer* serializer = new Asn1BerSerializer(node, &subject);
    
    serializer->serializeAllTree();
    
    QVERIFY( subject.size() == 10 );
    QVERIFY( (unsigned char) subject.at(0) == 0x30 ); // universal 16 (complex) - sequence (of)
    QVERIFY( (unsigned char) subject.at(1) == UNKNOWN_LENGTH );
    QVERIFY( (unsigned char) subject.at(2) == 0x01 ); // universal 1 primitive
    QVERIFY( (unsigned char) subject.at(3) == 0x01 ); // length = 1
    QVERIFY( subject.at(4) == 'a' ); // value
    QVERIFY( (unsigned char) subject.at(5) == 0x01 ); // universal 1 primitive
    QVERIFY( (unsigned char) subject.at(6) == 0x01 ); // length = 1
    QVERIFY( subject.at(7) == 'a' ); // value
    QVERIFY( subject.at(8) == 0 ); // end-of-contents
    QVERIFY( subject.at(9) == 0 );
}

void Asn1BerSerializerTest::cleanupTestCase()
{
    qDebug() << "cleanup";
}

// this thing defines a standard main function
QTEST_MAIN(Asn1BerSerializerTest)

#include "serializertest.moc"